@@ -199,7 +199,21 @@ class NuisanceParameterPull(object):
                 'required'    : False,
                 'type'        : int,
                 'default'     : 0
-            }                          
+            },
+            'no_cache'  : {
+                'abbr'        : None,
+                'description' : 'Cache existing result', 
+                'required'    : False,
+                'type'        : bool,
+                'default'     : False
+            },
+            'exclude'  : {
+                'abbr'        : None,
+                'description' : 'Exclude NPs (wildcard is accepted)', 
+                'required'    : False,
+                'type'        : str,
+                'default'     : ''
+            }        
         } 
     
     @property
@@ -276,11 +290,11 @@ class NuisanceParameterPull(object):
                    precision:float=0.001, eps:float=1.0, strategy:int=0, 
                    num_cpu:int=1, offset:int=1, optimize:int=2, eigen:bool=False,
                    fix_cache:bool=True, fix_multi:bool=True, max_calls:int=-1, 
-                   max_iters:int=-1, outdir:str='output', checkpoint=True, **kwargs):
+                   max_iters:int=-1, outdir:str='output', no_cache=False, **kwargs):
         outname_root = os.path.join(outdir, nuis_name + '.root')
         outname_json = os.path.join(outdir, nuis_name + '.json')
         #checkpointing:
-        if (os.path.exists(outname_root) and os.path.exists(outname_json)) and checkpoint:
+        if (os.path.exists(outname_root) and os.path.exists(outname_json)) and (not no_cache):
             print('INFO: Jobs already finished for the NP: {}. Skipping.'.format(nuis_name))
             return None
         start_time = time.time()
@@ -459,7 +473,7 @@ class NuisanceParameterPull(object):
                  data='combData', poi='', snapshot='nominalNuis', outdir='output', profile="",
                  fix='', minimizer_type='Minuit2', minimizer_algo='Migrad', num_cpu=1, binned=1,
                  precision=0.001, eps=1.0, log_level='INFO', eigen=0, strategy=0, fix_cache=1, fix_multi=1,
-                 offset=1, optimize=2, parameter="", max_calls=-1, max_iters=-1, parallel=0, checkpoint=True, **kwargs):
+                 offset=1, optimize=2, parameter="", max_calls=-1, max_iters=-1, parallel=0, no_cache=False, **kwargs):
         
         start_time = time.time()
         
@@ -469,15 +483,22 @@ class NuisanceParameterPull(object):
         
         
         nuis_list = ExtendedModel.get_nuisance_parameter_names(input_file, workspace, model_config)
-        nuis_patterns = parameter.split(',')
+        include_patterns = parameter.split(',')
         if parameter:
-            nuis_names = []
+            nuis_to_include = []
             for nuis_name in nuis_list:
                 # filter out nuisance parameters
-                if any(fnmatch.fnmatch(nuis_name, nuis_pattern) for nuis_pattern in nuis_patterns):
-                    nuis_names.append(nuis_name)
+                if any(fnmatch.fnmatch(nuis_name, include_pattern) for include_pattern in include_patterns):
+                    nuis_to_include.append(nuis_name)
         else:
-            nuis_names = nuis_list
+            nuis_to_include = nuis_list
+        exclude_patterns = exclude.split(',')
+        nuis_to_exclude = []
+        if exclude_patterns:
+            for nuis_name in nuis_list:
+                if any(fnmatch.fnmatch(nuis_name, exclude_pattern) for exclude_pattern in exclude_patterns):
+                    nuis_to_exclude.append(nuis_name)
+        nuis_names = list(set(nuis_to_include) - set(nuis_to_exclude))
                 
         if parallel:
             from itertools import repeat
